@@ -1,5 +1,6 @@
 from _ast import operator
 from abc import abstractmethod, ABC
+import copy
 import sys
 import random
 import math
@@ -7,13 +8,14 @@ from math import exp
 import numpy as np
 import functools
 import itertools
+from copy import deepcopy
 
-from .individual import Individual
 from .utils import VectorAndNumbers
 from .doe import build_box_behnken, build_lhs, build_full_fact, build_plackett_burman, build_gsd, build_halton
 from .job import Job
 from joblib import Parallel, delayed
-from copy import deepcopy
+from .individual import Individual
+
 
 EPSILON = sys.float_info.epsilon
 
@@ -193,10 +195,9 @@ class WorstCaseEvaluator(Evaluator):
 
 
 class Generator(Operator):
-    def __init__(self, parameters=None, features=dict()):
+    def __init__(self, parameters=None):
         super().__init__()
         self.parameters = parameters
-        self.features = features
 
     @abstractmethod
     def generate(self):
@@ -204,23 +205,45 @@ class Generator(Operator):
 
 
 class CustomGenerator(Generator):
-    def __init__(self, parameters=None, features=dict()):
-        super().__init__(parameters, features)
+    def __init__(self, parameters=None):
+        super().__init__(parameters)
         self.vectors = []
 
     def init(self, vectors):
         self.vectors = vectors
 
+    def generate(self):
+        vectors = []
+        for vector in self.vectors:
+            vectors.append(vector)
+        return vectors
+
+
+class PartiallyCustomGenerator(Generator):
+    def __init__(self, parameters=None):
+        super().__init__(parameters)
+        self.vectors = []
+        self.number = None
+
+    def init(self, vectors, number):
+        self.vectors = vectors
+        self.number = number
+
     def generate(self):
         individuals = []
         for vector in self.vectors:
-            individuals.append(Individual(vector, self.features))
-        return individuals
+            individuals.append(Individual(vector))
+        req = self.number - len(individuals)
+        vectors = []
+        for i in range(req + 1):
+            vector = VectorAndNumbers.gen_vector(self.parameters)
+            vectors.append(Individual(vector))
+        return vectors
 
 
 class UniformGenerator(Generator):
-    def __init__(self, parameters=None, features=dict()):
-        super().__init__(parameters, features)
+    def __init__(self, parameters=None):
+        super().__init__(parameters)
         self.number = 0
 
     def init(self, number):
@@ -235,26 +258,27 @@ class UniformGenerator(Generator):
             for i in range(self.number):
                 vectors[-1].append(parameter['bounds'][0] + i * delta)
 
+        result = []
         for combination in itertools.product(*vectors):
-            individuals.append(Individual(list(combination), self.features))
+            result.append(list(combination))
 
-        return individuals
+        return result
 
 
 class RandomGenerator(Generator):
-    def __init__(self, parameters=None, features=dict()):
-        super().__init__(parameters, features)
+    def __init__(self, parameters=None):
+        super().__init__(parameters)
         self.number = 0
 
     def init(self, number):
         self.number = number
 
     def generate(self):
-        individuals = []
+        vectors = []
         for i in range(self.number):
             vector = VectorAndNumbers.gen_vector(self.parameters)
-            individuals.append(Individual(vector, self.features))
-        return individuals
+            vectors.append(vector)
+        return vectors
 
 
 class IntegerGenerator(Generator):
@@ -262,8 +286,8 @@ class IntegerGenerator(Generator):
     Generate Integer vectors for problems with discrete values
     """
 
-    def __init__(self, parameters, features=dict()):
-        super().__init__(parameters, features)
+    def __init__(self, parameters):
+        super().__init__(parameters)
         # self.parameters['parameter_type'] = 'integer'
         self.number = 0
 
@@ -274,11 +298,11 @@ class IntegerGenerator(Generator):
                 parameter['parameter_type'] = 'integer'
 
     def generate(self):
-        individuals = []
+        vectors = []
         for i in range(self.number):
             vector = VectorAndNumbers.gen_vector(self.parameters)
-            individuals.append(Individual(vector, self.features))
-        return individuals
+            vectors.append(vector)
+        return vectors
 
 
 class FullFactorGenerator(Generator):
@@ -287,8 +311,8 @@ class FullFactorGenerator(Generator):
     Number of experiments (2 ** len(parameters) - without center, 3 ** len(parameters - with center)
     """
 
-    def __init__(self, parameters=None, features=dict()):
-        super().__init__(parameters, features)
+    def __init__(self, parameters=None):
+        super().__init__(parameters)
         self.center = False
 
     def init(self, center):
@@ -307,11 +331,7 @@ class FullFactorGenerator(Generator):
                 dict_vars[name] = [l_b, u_b]
 
         df = build_full_fact(dict_vars)
-
-        individuals = []
-        for vector in df:
-            individuals.append(Individual(vector, self.features))
-        return individuals
+        return df
 
 
 class FullFactorLevelsGenerator(Generator):
@@ -320,8 +340,8 @@ class FullFactorLevelsGenerator(Generator):
     Number of experiments (2 ** len(parameters) - without center, 3 ** len(parameters - with center)
     """
 
-    def __init__(self, parameters=None, features=dict()):
-        super().__init__(parameters, features)
+    def __init__(self, parameters=None):
+        super().__init__(parameters)
         self.values = []
 
     def init(self, values):
@@ -333,11 +353,7 @@ class FullFactorLevelsGenerator(Generator):
             dict_vars[parameter['name']] = value
 
         df = build_full_fact(dict_vars)
-
-        individuals = []
-        for vector in df:
-            individuals.append(Individual(vector, self.features))
-        return individuals
+        return df
 
 
 class PlackettBurmanGenerator(Generator):
@@ -346,8 +362,8 @@ class PlackettBurmanGenerator(Generator):
     Number of experiments (2 ** len(parameters) - without center, 3 ** len(parameters - with center)
     """
 
-    def __init__(self, parameters=None, features=dict()):
-        super().__init__(parameters, features)
+    def __init__(self, parameters=None):
+        super().__init__(parameters)
 
     def generate(self):
         dict_vars = {}
@@ -358,12 +374,7 @@ class PlackettBurmanGenerator(Generator):
             dict_vars[name] = [l_b, u_b]
 
         df = build_plackett_burman(dict_vars)
-        # print(df)
-
-        individuals = []
-        for vector in df:
-            individuals.append(Individual(vector, self.features))
-        return individuals
+        return df
 
 
 class BoxBehnkenGenerator(Generator):
@@ -378,8 +389,8 @@ class BoxBehnkenGenerator(Generator):
     https://en.wikipedia.org/wiki/Box%E2%80%93Behnken_design
     """
 
-    def __init__(self, parameters=None, features=dict()):
-        super().__init__(parameters, features)
+    def __init__(self, parameters=None):
+        super().__init__(parameters)
 
     def generate(self):
         dict_vars = {}
@@ -391,11 +402,7 @@ class BoxBehnkenGenerator(Generator):
 
         df = build_box_behnken(dict_vars)
         # print(df)
-
-        individuals = []
-        for vector in df:
-            individuals.append(Individual(vector, self.features))
-        return individuals
+        return df
 
 
 class LHSGenerator(Generator):
@@ -403,8 +410,8 @@ class LHSGenerator(Generator):
     Builds a Latin Hypercube design dataframe from a dictionary of factor/level ranges.
     """
 
-    def __init__(self, parameters=None, features=dict()):
-        super().__init__(parameters, features)
+    def __init__(self, parameters=None):
+        super().__init__(parameters)
         self.number = 0
 
     def init(self, number):
@@ -420,11 +427,7 @@ class LHSGenerator(Generator):
 
         df = build_lhs(dict_vars, num_samples=self.number)
         # print(df)
-
-        individuals = []
-        for vector in df:
-            individuals.append(Individual(vector, self.features))
-        return individuals
+        return df
 
 
 class HaltonGenerator(Generator):
@@ -432,8 +435,8 @@ class HaltonGenerator(Generator):
     Builds a Halton sequence.
     """
 
-    def __init__(self, parameters=None, features=dict()):
-        super().__init__(parameters, features)
+    def __init__(self, parameters=None):
+        super().__init__(parameters)
         self.number = 0
 
     def init(self, number):
@@ -448,11 +451,7 @@ class HaltonGenerator(Generator):
             dict_vars[name] = [l_b, u_b]
 
         df = build_halton(dict_vars, num_samples=self.number)
-
-        individuals = []
-        for vector in df:
-            individuals.append(Individual(vector, self.features))
-        return individuals
+        return df
 
 
 class GSDGenerator(Generator):
@@ -460,8 +459,8 @@ class GSDGenerator(Generator):
     Builds a Latin Hypercube design dataframe from a dictionary of factor/level ranges.
     """
 
-    def __init__(self, parameters=None, features=dict()):
-        super().__init__(parameters, features)
+    def __init__(self, parameters=None):
+        super().__init__(parameters)
         self.values = []
         self.reduction = 1
         self.n = 1
@@ -479,13 +478,13 @@ class GSDGenerator(Generator):
         df = build_gsd(levels, self.reduction, self.n)
         # print(df)
 
-        individuals = []
+        vectors = []
         for vector in df:
             vals = []
             for i in range(len(vector)):
                 vals.append(self.values[i][vector[i]])
-            individuals.append(Individual(vals, self.features))
-        return individuals
+            vectors.append(vals)
+        return vectors
 
 
 class Mutator(Operator):
@@ -520,7 +519,7 @@ class SimpleMutator(Mutator):
             else:
                 vector.append(p.vector[i])
 
-        p_new = Individual(vector, p.features)
+        p_new = vector
         return p_new
 
 
@@ -543,11 +542,11 @@ class PmMutator(Mutator):
             if random.uniform(0, 1) < self.probability:
                 l_b = parameter['bounds'][0]
                 u_b = parameter['bounds'][1]
-                vector.append(self.pm_mutation(parent.vector[i], l_b, u_b))
+                vector.append(self.pm_mutation(parent[i], l_b, u_b))
             else:
-                vector.append(parent.vector[i])
+                vector.append(parent[i])
 
-        return Individual(vector, parent.features)
+        return vector
 
     def pm_mutation(self, x, lb, ub):
         """
@@ -602,11 +601,11 @@ class UniformMutator(Mutator):
             if random.uniform(0, 1) < self.probability:
                 l_b = parameter['bounds'][0]
                 u_b = parameter['bounds'][1]
-                vector.append(self.uniform_mutation(parent.vector[i], l_b, u_b))
+                vector.append(self.uniform_mutation(parent[i], l_b, u_b))
             else:
-                vector.append(parent.vector[i])
+                vector.append(parent[i])
 
-        return Individual(vector, parent.features)
+        return vector
 
     def uniform_mutation(self, x, lb, ub):
 
@@ -634,11 +633,11 @@ class NonUniformMutation(Mutator):
             if random.uniform(0, 1) < self.probability:
                 l_b = parameter['bounds'][0]
                 u_b = parameter['bounds'][1]
-                vector.append(self.non_uniform_mutation(parent.vector[i], l_b, u_b, current_iteration))
+                vector.append(self.non_uniform_mutation(parent[i], l_b, u_b, current_iteration))
             else:
-                vector.append(parent.vector[i])
+                vector.append(parent[i])
 
-        return Individual(vector, parent.features)
+        return vector
 
     def non_uniform_mutation(self, x, lb, ub, current_iteration):
 
@@ -1017,9 +1016,11 @@ class EpsilonDominance(Dominance):
         for i, (p_costs, q_costs) in enumerate(zip(p[:-1], q[:-1])):
 
             epsilon = float(self.epsilons[i % len(self.epsilons)])
+            if epsilon == 0:
+                epsilon = 1e-3
 
-            p_eps = math.floor(p_costs / epsilon)
-            q_eps = math.floor(q_costs / epsilon)
+            p_eps = p_costs / epsilon
+            q_eps = q_costs / epsilon
 
             if p_eps > q_eps:
                 dominate_q = True
@@ -1037,8 +1038,8 @@ class EpsilonDominance(Dominance):
             for i, (p_costs, q_costs) in enumerate(zip(p[:-1], q[:-1])):
                 epsilon = float(self.epsilons[i % len(self.epsilons)])
 
-                i1 = math.floor(p_costs / epsilon)
-                i2 = math.floor(q_costs / epsilon)
+                i1 = p_costs / epsilon
+                i2 = q_costs / epsilon
 
                 dist1 += math.pow(p_costs - i1 * epsilon, 2.0)
                 dist2 += math.pow(q_costs - i2 * epsilon, 2.0)
@@ -1164,19 +1165,19 @@ class Selector(Operator):
 
         return None
 
-    def fast_nondominated_sorting(self, population):
+    def fast_nondominated_sorting(self, individuals):
         pareto_front = [[]]
         front_number = 1
 
         # reset elements
-        for p in population:
-            p.features['domination_counter'] = 0
-            p.features['front_number'] = None
-            p.features['dominate'] = []
+        for individual in individuals:
+            individual.features['domination_counter'] = 0
+            individual.features['front_number'] = None
+            individual.features['dominate'] = []
 
-        for i, p in enumerate(population):
-            for j in range(i + 1, len(population)):
-                q = population[j]
+        for i, p in enumerate(individuals):
+            for j in range(i + 1, len(individuals)):
+                q = individuals[j]
                 dom = self.comparator.compare(p.costs_signed, q.costs_signed)
                 if dom == 1:
                     p.features['dominate'].append(q.id)
@@ -1195,7 +1196,7 @@ class Selector(Operator):
             pareto_front.append([])
             for p in pareto_front[front_number - 2]:
                 for individual_id in p.features['dominate']:
-                    q = self.individual(population, individual_id)
+                    q = self.individual(individuals, individual_id)
                     q.features['domination_counter'] -= 1
                     if q.features['domination_counter'] == 0 and q.features['front_number'] is None:
                         q.features['front_number'] = front_number
@@ -1206,8 +1207,6 @@ class Selector(Operator):
         for sub_front in pareto_front:
             crowding_distance(sub_front)
 
-        return
-
 
 def crowding_distance(front):
     """
@@ -1262,6 +1261,7 @@ def nondominated_truncate(population, size):
     # calculating the crowding distance on the different fronts
     population = list(set(population))
     result = sorted(population, key=functools.cmp_to_key(nondominated_cmp))
+    # ToDo:  Return also individuals which are not included in next population , result[size:]
     return result[:size]
 
 
@@ -1306,10 +1306,8 @@ class CopySelector(Selector):
     def select(self, individuals):
         selection = []
         for individual in individuals:
-            candidate = Individual(individual.vector)
-            candidate.costs = deepcopy(individual.costs)
+            candidate = individual.copy()
             candidate.features = deepcopy(individual.features)
-            candidate.population_id = -1
             selection.append(candidate)
         return selection
 
@@ -1433,8 +1431,8 @@ class SimpleCrossover(Crossover):
         super().__init__(parameters, probability)
 
     def cross(self, p1, p2):
-        parent_a = p1.vector.copy()
-        parent_b = p2.vector.copy()
+        parent_a = p1.copy()
+        parent_b = p2.copy()
 
         if random.uniform(0.0, 1.0) <= self.probability:
             parameter1 = []
@@ -1453,8 +1451,8 @@ class SimpleCrossover(Crossover):
             parent_a = parameter1
             parent_b = parameter2
 
-        offspring_a = Individual(parent_a, p1.features)
-        offspring_b = Individual(parent_b, p2.features)
+        offspring_a = parent_a
+        offspring_b = parent_b
 
         return offspring_a, offspring_b
 
@@ -1479,8 +1477,10 @@ class SimulatedBinaryCrossover(Crossover):
         a list with 2 offsprings each with the genotype of an  offspring after recombination and mutation.
         """
 
-        x1 = deepcopy(p1.vector)
-        x2 = deepcopy(p2.vector)
+        # Make new instance of class derived from Individual
+
+        x1 = p1.copy()
+        x2 = p2.copy()
 
         if random.random() <= self.probability:
             for i, param in enumerate(self.parameters):
@@ -1527,7 +1527,7 @@ class SimulatedBinaryCrossover(Crossover):
                         else:
                             x1[i], x2[i] = c1, c2
 
-        return Individual(x1, p1.features), Individual(x2, p2.features)
+        return x1, x2
 
 
 class Normalization(Operator):
@@ -1696,4 +1696,55 @@ class MaximumFunctionCallTermination(Termination):
             self.n_max_evals = float("inf")
 
     def _do_continue(self, algorithm, **kwargs):
-        return algorithm.evaluator.n_eval < self.n_max_evals
\ No newline at end of file
+        return algorithm.evaluator.n_eval < self.n_max_evals
+
+
+def derivative(problem, points):
+    """
+        Compute derivative of func at points using finite differences
+
+               ddx = \\frac{func(points + eps) - func(points - eps)}{2 * eps}
+
+            Args:
+                problem (function): function with N parameters
+                points (array): array with N-dimension
+            Returns:
+                derivative: list with derivative
+                for instance:
+                    derivative[0] = d/d X1 func evaluated at points[0]
+                    derivative[1] = d/d X2 func evaluated at points[1]
+        """
+
+    d = []
+    eps = 1e-4
+    for i, p in enumerate(points):
+        step_up, step_down = list(points), list(points)  # copy list
+
+        # approximate derivative by tangent line with eps distance at
+        step_up[i] = p + eps
+        step_down[i] = p - eps
+        step_up = Individual(step_up)
+        step_down = Individual(step_down)
+        d.append((problem.evaluate(step_up)[0] - problem.evaluate(step_down)[0]) / (2 * eps))
+        # d.append((res1 - res2) / (2 * eps))
+
+    return d
+
+
+def std_linear(grad_g_x, sig):
+    """
+        Compute the standard deviation of g(Xi) using linear approximation
+                . math::
+                    sig_g = (sum_i^n (grad_g_xi * sig_Xi)^2)^(1/2)
+            Args:
+                grad_g_x (array): partial derivative of g with respect xi
+                sig (array): standard deviation of variables
+            Returns:
+                std_g (float): standard deviation of g(Xi)
+        """
+
+    std_g = 0
+    for dg_dx, s in zip(grad_g_x, sig):
+        std_g += (dg_dx ** 2 * s ** 2)
+    std_g = std_g ** (1 / 2)
+    return std_g
