@@ -105,7 +105,8 @@ class Problem:
         self.output_files = None
 
         self.working_dir = tempfile.gettempdir() + os.sep + "artap-{}".format(ts) + os.sep
-        os.mkdir(self.working_dir)
+        if not os.path.exists(self.working_dir):
+            os.mkdir(self.working_dir)
 
         # surrogate model (default - only simple eval)
         self.surrogate = SurrogateModelEval(self)
@@ -124,6 +125,9 @@ class Problem:
         # clean up
         atexit.register(self.cleanup)
 
+    def __del__(self):
+        pass
+
     def populations(self):
         individuals = {}
         for individual in self.individuals:
@@ -208,7 +212,6 @@ class Problem:
         """ :param individual: Individual """
         pass
 
-    @abstractmethod
     def evaluate_inequality_constraints(self, x):
         """ :param x: vector """
         return []
