@@ -1,5 +1,8 @@
 import csv
+import numpy as np
 from .quality_indicator import gd, epsilon_add
+from .operators import derivative, std_linear
+from .individual import Individual
 
 
 class Results:
@@ -9,18 +12,22 @@ class Results:
         self.problem = problem
 
     def parameter_names(self):
+        """ Returns a list of parameter names for the given problem. """
         parameter_names = []
         for parameter in self.problem.parameters:
             parameter_names.append(parameter["name"])
         return parameter_names
 
     def parameter_number(self):
+        """ Returns the number of parameters."""
         return len(self.problem.parameters)
 
     def goal_number(self):
+        """ Returns the number of goal functions. """
         return len(self.problem.costs)
 
     def goal_names(self):
+        """ Returns a list of goal names for the given problem. """
         cost_names = []
         for cost in self.problem.costs:
             cost_names.append(cost["name"])
@@ -40,7 +47,6 @@ class Results:
         else:
             raise ValueError('There is not a cost function with given name: {}'.format(name))
 
-
     def parameters(self):
         out = []
         for individuals in self.problem.populations().values():
@@ -49,13 +55,23 @@ class Results:
         return out
 
     def costs(self):
+        """ Returns the list containing cost values for all individuals.
+            Example of usage:
+                costs = results.costs()
+                plt.plot(costs, '.')
+        """
         out = []
-        for individuals in self.problem.populations().values():
-            for individual in individuals:
-                out.append(individual.costs)
+        n = len(self.problem.individuals[0].costs)
+        for i in range(n):
+            out.append([])
+
+        for individual in self.problem.individuals:
+            for i in range(n):
+                out[i].append(individual.costs[i])
         return out
 
     def table(self, transpose=True):
+        """ Returns the table where rows consist of values of parameters and values of goal functions. """
         out = []
         for individuals in self.problem.populations().values():
             for individual in individuals:
@@ -249,8 +265,10 @@ class Results:
         pareto_front = []
         for i in range(n):
             pareto_front.append([])
-            for individual in individuals:
-                if individual.features['front_number'] == 1:
+
+        for individual in individuals:
+            if individual.features['front_number'] == 1:
+                for i in range(n):
                     pareto_front[i].append(individual.costs[i])
 
         return pareto_front
@@ -271,7 +289,6 @@ class Results:
         :return:
         """
 
-
         # get the index of the required parameter
         index = 0  # default - one parameter
         min_l = []
@@ -293,7 +310,6 @@ class Results:
         opt = min(min_l, key=lambda x: x.costs[index])
         return opt
 
-
     # TODO: same function - David, could you check it and write test?
     def pareto_values(self, archive=None):
         """
@@ -336,9 +352,106 @@ class Results:
 
         return result
 
-
     def get_population_ids(self):
         ids = set()
         for individual in self.problem.individuals:
             ids.add(individual.population_id)
         return ids
+
+    def integration_measure(self):
+        Integration = 0
+        opt = self.find_optimum('f_1')
+        for individual in self.problem.individuals:
+            Integration += individual.costs[0]
+        ub = self.problem.u_b
+        lb = self.problem.l_b
+        sampling_size = self.problem.sampling_size
+        result = ((ub - lb) / len(self.problem.individuals)) * Integration
+
+        return result
+
+    def form_reliability(self, problem, individual):
+        """
+            Performs the FORM-HLRF algorithm
+                The goal is to find the reliability index for a nonlinear function g(Xi).
+
+                HL_RF algorithm steps:
+                1) Define the limit state function (𝑔) and the convergence criterion (tol = 1e-3), statistical characteristics
+                 of basic random variables (mean, standard deviation, and distribution function for each random variable).
+                2) Transfer the random variables from 𝑋-space to 𝑈-space according to
+                    .. math::
+                        U = (X - \mu_X^{e}) / \sigma_X^{e}
+                3) Compute the gradient vector and limit state function at point U𝑘.
+                4) Compute the cost function using
+                    .. math::
+                        U^{k+1} = [(G(U^{k}) - \grad G(U^{k})U^{k}) / \norm{\grad G(U^{k})}] \alpha^{k}
+
+                    where \alpha^{k} is sensivity factor that can be calculated by:
+                    ..math::
+                        \alpha^{k} = -(\grad G(U^{k})) / \norm{\grad G(U^{k})}
+
+                5) Transfer the basic random variables from 𝑈-space to 𝑋-space.
+                6) Control the convergence conditions as below; if true then the method is converged, stop and go to step (7);
+                else, 𝑘 = 𝑘+1 and go to step (2).
+                7) Estimate the failure probability by
+                    ..math::
+                        P_f ~~ \phi(-\beta)
+                    where \phi is the CDF of the standard normal distribution and \beta = \norm{U^{*}} is the reliability index.
+
+                ..Reference::
+
+                [1] A Novel Algorithm for Structural Reliability Analysis Based on Finite Step Length and Armijo Line Search
+                    https://www.mdpi.com/2076-3417/9/12/2546
+                [2] A Comparative Study of First-Order Reliability Method-Based Steepest Descent Search Directions for
+                 Reliability Analysis of Steel Structures, https://www.hindawi.com/journals/ace/2017/8643801/
+                [3] Reliability analysis and optimal design under uncertainty - Focus on adaptive surrogate-based approaches
+                    https://tel.archives-ouvertes.fr/tel-01737299/document
+
+            Args:
+                problem
+                individual which is optimal
+            Return:
+                vector: converged design point
+                reliability_index
+        """
+        convergence = False
+        tol = 1e-3
+        mean = np.random.uniform(0, 10, len(problem.parameters))
+        std = np.random.uniform(0, 10, len(problem.parameters))
+
+        mu_g = problem.evaluate(individual)
+        grad_g_x = np.array(derivative(problem, mean))
+        sig_g = std_linear(grad_g_x, std)
+        # reliability_index
+        reliability_index = mu_g / sig_g
+        alpha = - grad_g_x * std / sig_g
+
+        # Initialize design points
+        vector = mean + reliability_index * std * alpha
+
+        while not convergence:
+            transform_individuals = (vector - mean) / std
+
+            # compute gradient of g with respect to z
+            grad_g_x = derivative(problem, vector)
+            grad_g_z = grad_g_x * std
+
+            beta_previous = reliability_index
+            reliability_index = (problem.evaluate(Individual(vector)) - grad_g_z @ transform_individuals) \
+                                / np.linalg.norm(grad_g_z)
+            alpha = - grad_g_z / np.linalg.norm(grad_g_z)
+
+            # update design points in standard space
+            transform_individuals = alpha * reliability_index
+            # transform to physical space
+            physical_transform = mean + transform_individuals * std
+
+            condition1 = np.linalg.norm(physical_transform - vector) \
+                         / np.linalg.norm(physical_transform) < tol
+            condition2 = abs(np.round(reliability_index, 3) - np.round(beta_previous, 3)) < tol
+            if condition2 or condition1:
+                convergence = True
+            if not convergence:
+                vector = physical_transform
+
+        return vector, reliability_index
