@@ -13,7 +13,7 @@ class TestAckleyOMOPSO(unittest.TestCase):
         problem = Ackley(**{'dimension': 1})
         algorithm = OMOPSO(problem)
         algorithm.options['max_population_number'] = population_number
-        algorithm.options['max_population_size'] = 100
+        algorithm.options['max_population_size'] = 40
         algorithm.options['max_processes'] = 10
         algorithm.options['epsilons'] = 0.1
         algorithm.run()
@@ -28,10 +28,10 @@ class TestZDT1OMOPSO(unittest.TestCase):
     # integration test -- tests the total functionality of OMOPSO
 
     def test_local_problem(self):
-        problem = problem = ZDT1()
+        problem = ZDT1()
         algorithm = OMOPSO(problem)
-        algorithm.options['max_population_number'] = 500
-        algorithm.options['max_population_size'] = 100  # according to the literature
+        algorithm.options['max_population_number'] = 100
+        algorithm.options['max_population_size'] = 200  # according to the literature
         algorithm.options['max_processes'] = 1
         algorithm.options['epsilons'] = 0.05
         algorithm.run()
@@ -63,9 +63,9 @@ class TestZDT1SMPSP(unittest.TestCase):
     # integration test -- tests the total functionality of SMPSO
 
     def test_local_problem(self):
-        problem = problem = ZDT1()
+        problem = ZDT1()
         algorithm = OMOPSO(problem)
-        algorithm.options['max_population_number'] = 500
+        algorithm.options['max_population_number'] = 200
         algorithm.options['max_population_size'] = 100  # according to the literature
         algorithm.options['max_processes'] = 1
         algorithm.run()
@@ -87,6 +87,7 @@ class TestRosenbrockSMPSO(unittest.TestCase):
         algorithm.run()
 
         result = Results(problem)
+
         optimum = result.find_optimum('f_1')
         print(optimum.costs[0])
         print(problem.global_optimum)
@@ -184,7 +185,7 @@ class TestAckleyPSOGA(unittest.TestCase):
         problem = Ackley(**{'dimension': 1})
         algorithm = PSOGA(problem)
         algorithm.options['max_population_number'] = population_number
-        algorithm.options['max_population_size'] = 100
+        algorithm.options['max_population_size'] = 50
         algorithm.options['max_processes'] = 10
         algorithm.run()
 
