@@ -2,12 +2,12 @@ import unittest
 import os
 import tempfile
 import csv
+import numpy as np
 
-from matplotlib.pyplot import show
-
+from ..algorithm_monte_carlo import Numerical_Integrator
 from ..problem import Problem
 from ..algorithm_sweep import SweepAlgorithm
-from ..algorithm_genetic import NSGAII
+from ..algorithm_NSGAII import NSGAII
 from ..operators import CustomGenerator
 from ..results import Results
 
@@ -44,7 +44,8 @@ class TestGearDesignProblem(Problem):
                       {'name': 'f_2', 'criteria': 'minimize'}]
 
     def evaluate(self, individual):
-        f1 = (1. / 6.931 - (individual.vector[0] * individual.vector[1]) / (individual.vector[2] * individual.vector[3])) ** 2.
+        f1 = (1. / 6.931 - (individual.vector[0] * individual.vector[1]) / (
+                individual.vector[2] * individual.vector[3])) ** 2.
         f2 = max(individual.vector)
         return [f1, f2]
 
@@ -52,6 +53,30 @@ class TestGearDesignProblem(Problem):
         pass
 
 
+class IntegralProblem(Problem):
+
+    def set(self, **kwargs):
+        """Time-dependent 1D QM wave function of a single particle - squared."""
+        # self.set_dimension(**kwargs)
+        # self.parameters = self.generate_paramlist(self.dimension, lb=0.0, ub=1.0)
+        self.parameters = [{'name': 'x1', 'bounds': [0, 1], 'parameter_type': 'integer'}]
+
+        self.u_b = 3 * np.pi / 2
+        self.l_b = 0
+        self.sampling_size = 100
+        self.integral_global = 1.0
+
+        # single objective problem
+        self.costs = [{'name': 'f_1', 'criteria': 'minimize'}]
+
+    def evaluate(self, z):
+        x = z.vector
+        # output = 0
+        # for c in x:
+        f_1 = np.sin(x[0])
+        return [f_1]
+
+
 class TestResults(unittest.TestCase):
     """ Tests simple one objective optimization problem."""
 
@@ -108,10 +133,10 @@ class TestResults(unittest.TestCase):
 
     def test_costs(self):
         costs = self.results.costs()
-        self.assertEqual(len(costs), 3)
+        self.assertEqual(len(costs[0]), 3)
         self.assertEqual(costs[0][0], 20.0)
-        self.assertEqual(costs[1][0], 10.0)
-        self.assertEqual(costs[2][0], 20.0)
+        self.assertEqual(costs[0][1], 10.0)
+        self.assertEqual(costs[0][2], 20.0)
 
     def test_table(self):
         table = self.results.table(transpose=False)
@@ -229,3 +254,15 @@ class TestResults(unittest.TestCase):
         individuals = self.results.population()
         self.assertEqual(len(individuals), 3)
         self.assertEqual(individuals[1].costs[0], 10.0)
+
+    def test_integration_measure(self):
+        problem = IntegralProblem(**{'dimension': 1})
+        algorithm = Numerical_Integrator(problem)
+        algorithm.options['max_population_number'] = 50
+        algorithm.options['max_population_size'] = 100
+        algorithm.options['max_processes'] = 10
+        algorithm.run()
+
+        result = Results(problem)
+        integral = result.integration_measure()
+        self.assertAlmostEqual(integral, problem.integral_global, places=1)
