@@ -28,7 +28,7 @@ class TestSALibSensitivity(unittest.TestCase):
         problem.options['save_data_files'] = True
 
         algorithm = SALibAlgorithm(problem)
-        algorithm.options["samples"] = 1024
+        algorithm.options["samples"] = 10000
         algorithm.options["method"] = "sobol"
         algorithm.options["print_to_console"] = True
         algorithm.run()
@@ -37,8 +37,4 @@ class TestSALibSensitivity(unittest.TestCase):
 
         self.assertAlmostEqual(res["S1"][0], 0.316831, 2)
         self.assertAlmostEqual(res["S1"][1], 0.443764, 2)
-        self.assertAlmostEqual(res["S1"][2], 0.012203, 2)
-
-
-if __name__ == '__main__':
-    unittest.main()
+        self.assertAlmostEqual(res["S1"][2], 0.002807, 2)
